import Flutter
import UIKit
import ENChatBot

public class SwiftEnBotSdkPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "en_bot_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftEnBotSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    var flutterResult : FlutterResult?
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "launchBot" :
            if let arguments = call.arguments as? [String : Any] {
                let botKey = arguments["botKey"] as? String ?? ""
                let botName = arguments["botName"] as? String ?? ""
                let brandingKey = arguments["brandingKey"] as? String ?? ""
                let welcomeMsg = arguments["welcomeMsg"] as? Bool ?? false
                let enableLog = arguments["enableLog"] as? Bool ?? false
                let userId = arguments["userId"] as? String ?? ""
                let chatHistorySize = arguments["chatHistorySize"] as? Int ?? 100
                let showDone = arguments["showDone"] as? Bool ?? false
                var closeButtonImage : UIImage?
                if let fdata = arguments["closeButtonImage"] as? FlutterStandardTypedData{
                    closeButtonImage = UIImage(data: fdata.data)
                }
                let userAttributes = arguments["userAttributes"] as? [String : Any] ?? [String: Any]()
//                let language = (arguments["language"] as? Int) ?? 0
                let headerTitleFont = arguments["headerTitleFont"] as? String ?? ""
                let headerDescriptionFont = arguments["headerDescriptionFont"]  as? String ?? ""
                var sendMessageButtonIcon : UIImage?
                if let fdata = arguments["sendMessageButtonIcon"] as? FlutterStandardTypedData{
                    sendMessageButtonIcon = UIImage(data: fdata.data)
                }
                DispatchQueue.main.async {
                    self.loadFont(fontName: headerTitleFont)
                    self.loadFont(fontName: headerDescriptionFont)
                    
                    DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+0.5) {
                        
                        let config = ENChatBotConfig(
                            botKey: botKey,
                            botName: botName,
                            brandingKey: brandingKey,
                            welcomeMsg: welcomeMsg,
                            enableLog: enableLog,
                            userId: userId,
                            chatHistorySize: chatHistorySize,
                            showDone: showDone,
                            closeButtonImage: closeButtonImage ?? nil,
                            language: ENChatBotConfig.ENChatBotLanguage.default,
                            headerTitleFont: headerTitleFont != "" ? UIFont(name: String(headerTitleFont.replacingOccurrences(of: "fonts/", with: "").split(separator: ".").first ?? ""), size: 14) : nil,
                            headerDescriptionFont: headerDescriptionFont != "" ? UIFont(name: String(headerDescriptionFont.replacingOccurrences(of: "fonts/", with: "").split(separator: ".").first ?? ""), size: 12) : nil ?? nil,
                            sendMessageButtonIcon:sendMessageButtonIcon ?? nil,
                            userAttributes: userAttributes)
                        let botVC = ENChatBotViewController(chatConfig: config, callBackDelegate: self)
                        let controller : FlutterViewController = UIApplication.shared.keyWindow?.rootViewController as! FlutterViewController
                        let nav = UINavigationController(rootViewController: botVC)
                        nav.modalPresentationStyle = .fullScreen
                        controller.present(nav, animated: true)
                        self.flutterResult = result
                    }
                }
                
               
                
            }
            
        default:
            break
        }
    }
    
    func loadFont(fontName: String) {
        let controller : FlutterViewController = UIApplication.shared.keyWindow?.rootViewController as! FlutterViewController
        let bundle = Bundle.main
        let fontKey = controller.lookupKey(forAsset: fontName)
        if let path = bundle.path(forResource: fontKey, ofType: nil) {            
            if let fontData = NSData(contentsOfFile: path ) {
                let dataProvider = CGDataProvider(data: fontData)
                let fontRef = CGFont(dataProvider!)
                var errorRef: Unmanaged<CFError>? = nil
                if let fr = fontRef {
                    if CTFontManagerRegisterGraphicsFont(fr, &errorRef) == false {
                        print("Failed to load font with error: \(errorRef.debugDescription)")
                    }
                }
            }
        }
        
    }
    
}

extension SwiftEnBotSdkPlugin : ENChatBotCallBack {
    public func didCloseENChatBot(_ withInfo: Any?) {
        flutterResult?(withInfo)
    }
}
